using System;
using System.IO;
using System.Text;
using DaveSexton.DocProject;

namespace SandcastleDocumentation
{
    public class CustomTopics
    {
        public static int Import(BuildContext context, string target)
        {
            int count;
            string rootFolder = context.ProjectDirectory;
            string sourceFolder = Path.Combine(rootFolder, "Custom Topics");
            string targetFolder = Path.Combine(rootFolder, target);

            count = CopyAllFilesInSourceToTarget(sourceFolder, targetFolder);

            MergeTOC(rootFolder, sourceFolder, "*.hhc");
            MergeTOC(rootFolder, sourceFolder, "*.hhk");
            return count;
        }


        private static void MergeTOC(string rootFolder, string sourceFolder, string fileExtension)
        {
            System.Diagnostics.Debug.Write("---MergeTOC---");

            string[] projectTOCFiles = Directory.GetFiles(rootFolder, fileExtension);
            string[] sourceTOCFile = Directory.GetFiles(sourceFolder, fileExtension);

            foreach(string tocPath in projectTOCFiles)
            {
                System.Diagnostics.Debug.Write(tocPath);

                StreamReader sr = new StreamReader(tocPath);
                string contents = sr.ReadToEnd();
                sr.Close();

                if (sourceTOCFile.Length > 0)
                {
                    StreamReader sourceSr = new StreamReader(sourceTOCFile[0]);
                    string newTOC = sourceSr.ReadToEnd();
                    sourceSr.Close();

                    int indexOf = contents.IndexOf("<UL>") + 4;

                    contents = contents.Insert(indexOf, newTOC);
                    System.Diagnostics.Debug.Write(newTOC);
                }

                StreamWriter sw = new StreamWriter(tocPath);
                sw.Write(contents);
                sw.Close();

                System.Diagnostics.Debug.Write("-----");
                System.Diagnostics.Debug.Write(tocPath);
            }
        }

        private static int CopyAllFilesInSourceToTarget(string sourceFolder, string targetFolder)
        {
            int count = 0;
            System.Diagnostics.Debug.Write("---CopyAllFilesInSourceToTarget---");
            System.Diagnostics.Debug.Write("Target Folder:" + targetFolder);

            if (!Directory.Exists(targetFolder))
                Directory.CreateDirectory(targetFolder);

            string[] files = Directory.GetFileSystemEntries(sourceFolder);

            foreach(string source in files)
            {
                string target = Path.Combine(targetFolder, Path.GetFileName(source));

                if (Directory.Exists(source)) //Must be a directory
                    count += CopyAllFilesInSourceToTarget(source, target);
                else //Must be a file
                {
                    if (Path.GetExtension(source).Equals(".htm"))
                    {
                        File.Copy(source, target, true);
                        count++;
                    }
                }
            }

            return count;
        }

        internal static int ImportHelp2(BuildContext context, string target)
        {
            //Content into Html2 folder
            //Table of Contents into .HxT

            throw new Exception("The method or operation is not implemented.");
        }
    }
}